@@ -202,7 +202,7 @@ class TestPO2OOCommand(test_convert.TestConvertCommand, TestPO2OO):
 
     def test_help(self, capsys):
         """tests getting help"""
-        options = test_convert.TestConvertCommand.test_help(self, capsys)
+        options = super().test_help(capsys)
         options = self.help_check(options, "--source-language=LANG")
         options = self.help_check(options, "--language=LANG")
         options = self.help_check(options, "-T, --keeptimestamp")
