@@ -36,11 +36,15 @@ class zh(common.Common):
 
     # Compared to common.py, we make the space after the sentence ending
     # optional and don't demand an uppercase letter to follow.
-    sentencere = re.compile(r"""(?s) # make . also match newlines
+    sentencere = re.compile(
+        r"""(?s) # make . also match newlines
                             .*?      # any text, but match non-greedy
                             [%s]     # the puntuation for sentence ending
                             \s*      # the optional space after the puntuation
-                            """ % sentenceend, re.VERBOSE)
+                            """
+        % sentenceend,
+        re.VERBOSE,
+    )
 
     # The following transformation rules should be mostly useful for all types
     # of Chinese. The comma (,) is not handled here, since it maps to two
@@ -67,5 +71,5 @@ class zh(common.Common):
         return 10 - length / 2
 
     ignoretests = {
-        'all': ["simplecaps", "startcaps"],
+        "all": ["simplecaps", "startcaps"],
     }
